unit FontUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ComCtrls, Grids, ExtCtrls;

type
  TFontForm = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    LBox: TListBox;
    Panel1: TPanel;
    PBox: TPaintBox;
    Panel2: TPanel;
    SbBold: TSpeedButton;
    SbItalic: TSpeedButton;
    SbUnderLine: TSpeedButton;
    SbStrikeOut: TSpeedButton;
    EdtSize: TEdit;
    UdSize: TUpDown;
    BtnSet: TBitBtn;
    BitBtn1: TBitBtn;
    BtnColor: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure LBoxClick(Sender: TObject);
    procedure EdtSizeChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure LBoxDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure SbBoldClick(Sender: TObject);
    procedure PBoxPaint(Sender: TObject);
    procedure BtnColorClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    FFontList: TStringList;
    FBitmap: TBitmap;
    FText: string;
    procedure   UpdateList();
    procedure   DrawSample();
    procedure   SetValues(F: TFont);

    procedure   WMFontChange(var Msg: TWMFontChange); message WM_FONTCHANGE;
  public
    { Public 錾 }
  end;

function ChooseFont(Owner: TComponent; var F: TFont): Boolean;

//var
//  FontForm: TFontForm;

implementation

uses
    ColorUnit, Misc;

{$R *.dfm}

(*
    -----------------------------------------------------------------
    tHgI_CAO
    -----------------------------------------------------------------
*)
function ChooseFont(Owner: TComponent; var F: TFont): Boolean;
var
    Form: TFontForm;
begin
    Result := false;
    Form := TFontForm.Create(Owner);
    Form.Left := TForm(Owner).Left + TForm(Owner).Width; 
    Form.Top := TForm(Owner).Top; 
    try
        Form.FBitmap.Canvas.Font.Assign(F);
        if (Form.ShowModal() = mrOK) then
        begin
            F.Assign(Form.FBitmap.Canvas.Font);
            Result := true;
        end;
    finally
        Form.Free;
    end;
end;

//------------------------------------------------------------------------
//  RXgN^
procedure TFontForm.FormCreate(Sender: TObject);
begin
    //  
    SetEditNumbersOnly(EdtSize);
    PBox.ControlStyle := PBox.ControlStyle + [csOpaque]; 
    //  IuWFNg쐬
    FFontList := TStringList.Create;
    FBitmap := TBitmap.Create;
    FBitmap.Width := PBox.Width;
    FBitmap.Height := PBox.Height;
    FBitmap.Canvas.Font.Assign(Self.Font);
end;

//------------------------------------------------------------------------
//  fXgN^
procedure TFontForm.FormDestroy(Sender: TObject);
begin
    FFontList.Free();
    FBitmap.Free;
end;

//------------------------------------------------------------------------
//  tHgꗗXV
procedure   TFontForm.UpdateList();
var
    OldSel: string;
    index: integer;
begin
    //  ̏ԂL^
    OldSel := '';
    if (LBox.ItemIndex >= 0) then
    begin
        OldSel := LBox.Items[LBox.ItemIndex];
    end;

    //  NA
    FFontList.Clear();

    //  XV
    EnumFontFamilies(Self.Canvas.Handle, nil, @FontEnumProc, LPARAM(FFontList));
    //  \[g
    FFontList.CustomSort(SortFontList);
    //  \
    LBox.Items.Assign(FFontList);

    //  łΕA
    if (OldSel <> '') then
    begin
        index := LBox.Items.IndexOf(OldSel);
        if (index <> -1) then
        begin
            LBox.ItemIndex := index;
        end;
    end
    else
    begin
        if (LBox.Count > 0) then LBox.ItemIndex := 0;
    end;
end;

//------------------------------------------------------------------------
//  tHgύXbZ[WLb`
procedure   TFontForm.WMFontChange(var Msg: TWMFontChange);
begin
    UpdateList();
    LBoxClick(LBox);
end;

//------------------------------------------------------------------------
//  tHg`
procedure TFontForm.LBoxDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
    LB: TListBox;
    C: TCanvas;
    RectT: TRect;
    RectH, MarginX, MarginY: integer;
    BackColor, ForeColor: TColor;
begin
    //  Zk̂߂̎o
    LB := TListBox(Control);
    C := LB.Canvas;

    //  ͈͂̍
    RectH := Rect.Bottom - Rect.Top;

    //  `ʒu
    MarginX := 3;
    MarginY := (RectH - C.TextHeight('Wg')) div 2;
    RectT := Rect;
    RectT.Left := RectT.Left + MarginX; 
    RectT.Top  := RectT.Top  + MarginY; 

    //  IĂ
    if (odSelected in State) then
    begin
        BackColor := RGB(010,036,106);
        ForeColor := RGB(255,255,255);
    end
    //  ȊO
    else
    begin
        BackColor := RGB(255,255,255);
        ForeColor := RGB(000,000,000);
    end;

    //  wihԂ
    C.Brush.Color := BackColor;
    C.Font.Color := ForeColor;
    C.FillRect(Rect);

    //  eLXg\
    C.TextRect(Rect, RectT.Left, RectT.Top, LB.Items[Index]);

    //  tH[JX_
    if (odFocused in State) then
    begin
        C.DrawFocusRect(Rect);
    end;
end;

//------------------------------------------------------------------------
procedure   TFontForm.SetValues(F: TFont);
var
    index: integer;
begin
    //  
    index := LBox.Items.IndexOf(F.Name);
    if (index <> -1) then LBox.ItemIndex := index;

    //  TCY
    UdSize.Position := F.Size;

    //  X^C
    SbBold.Down      := (fsBold in F.Style);
    SbItalic.Down    := (fsItalic in F.Style);
    SbUnderLine.Down := (fsUnderline in F.Style);
    SbStrikeOut.Down := (fsStrikeOut in F.Style);

    //  f
    LBoxClick(LBox);
end;

//------------------------------------------------------------------------
//  tHgύX
procedure TFontForm.LBoxClick(Sender: TObject);
var
    CS: TFontCharSet;
begin
    if (LBox.ItemIndex = -1) then exit;

    CS := TFontCharSet(LBox.Items.Objects[LBox.ItemIndex]);
    FText := SAMPLE_TEXT[0];
    if (CS = ANSI_CHARSET) then FText := SAMPLE_TEXT[0]
    else if (CS = SYMBOL_CHARSET) then FText := SAMPLE_TEXT[1]
    else if (CS = SHIFTJIS_CHARSET) then FText := SAMPLE_TEXT[2];

    FBitmap.Canvas.Font.Name := LBox.Items[LBox.ItemIndex];
    FBitmap.Canvas.Font.Charset := CS;

    DrawSample();
end;

//------------------------------------------------------------------------
//  X^CύX
procedure TFontForm.SbBoldClick(Sender: TObject);
var
    F: TFont;
begin
    F := FBitmap.Canvas.Font;
    F.Style := [];
    if (SbBold.Down)      then F.Style := F.Style + [fsBold];
    if (SbItalic.Down)    then F.Style := F.Style + [fsItalic];
    if (SbUnderLine.Down) then F.Style := F.Style + [fsUnderline];
    if (SbStrikeOut.Down) then F.Style := F.Style + [fsStrikeOut];

    DrawSample();
end;

//------------------------------------------------------------------------
//  TCYύX
procedure TFontForm.EdtSizeChange(Sender: TObject);
var
    E: TEdit;
begin
    E := TEdit(Sender);
    if (StrToIntDef(E.Text, 0) < 1) then E.Text := IntToStr(UdSize.Position);
    FBitmap.Canvas.Font.Size := StrToInt(E.Text);
    DrawSample();
end;

//------------------------------------------------------------------------
//  Tv`(ItXN[)
procedure   TFontForm.DrawSample();
var
    C: TCanvas;
    OffsetX, OffsetY: integer;
begin
    C := FBitmap.Canvas;
    C.Brush.Color := clWhite;
    C.FillRect(Rect(0, 0, FBitmap.Width, FBitmap.Height));
    OffsetX := (FBitmap.Width - C.TextWidth(FText)) div 2;
    OffsetY := (FBitmap.Height - C.TextHeight(FText)) div 2;
    C.TextOut(OffsetX, OffsetY, FText);
    PBox.Refresh();
end;

//------------------------------------------------------------------------
//  Tv`
procedure TFontForm.PBoxPaint(Sender: TObject);
begin
    BitBlt(PBox.Canvas.Handle, 0, 0, PBox.Width, PBox.Height,
            FBitmap.Canvas.Handle, 0, 0, SRCCOPY);
end;

//------------------------------------------------------------------------
//  FύX
procedure TFontForm.BtnColorClick(Sender: TObject);
var
    C: TColor;
begin
    C := FBitmap.Canvas.Font.Color;
    if (ChooseColor(Self, C)) then FBitmap.Canvas.Font.Color := C;
    DrawSample();
end;

//------------------------------------------------------------------------
//  \
procedure TFontForm.FormShow(Sender: TObject);
begin
    //  tHgꗗ\
    UpdateList();
    //  tHgݒ
    SetValues(FBitmap.Canvas.Font);
    EdtSize.OnChange := EdtSizeChange;
//    LBoxClick(LBox);
    //  Tv`
    DrawSample();
end;

end.

