unit LayerUnit;

interface

uses
  Windows, SysUtils, Classes, Graphics, Controls, Forms,
  ExtCtrls, StdCtrls, Buttons, Longo, MainUnit, ComCtrls;

type
  TLayerForm = class(TForm)
    Panel1: TPanel;
    BtnAdd: TBitBtn;
    BtnDel: TBitBtn;
    LBox: TListBox;
    Panel2: TPanel;
    BtnLT: TBitBtn;
    BtnCT: TBitBtn;
    BtnRT: TBitBtn;
    BtnLC: TBitBtn;
    BtnCC: TBitBtn;
    BtnRC: TBitBtn;
    BtnLB: TBitBtn;
    BtnCB: TBitBtn;
    BtnRB: TBitBtn;
    BtnUp: TBitBtn;
    BtnDown: TBitBtn;
    BtnCopy: TBitBtn;
    BtnFont: TBitBtn;
    BtnColor: TBitBtn;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    MemText: TMemo;
    Splitter1: TSplitter;
    EditLineSpan: TEdit;
    EditCharSpan: TEdit;
    UdCharSpan: TUpDown;
    UdLineSpan: TUpDown;
    procedure LBoxDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure BtnAddClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BtnDelClick(Sender: TObject);
    procedure BtnUpClick(Sender: TObject);
    procedure BtnDownClick(Sender: TObject);
    procedure LBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure LBoxMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BtnLTClick(Sender: TObject);
    procedure BtnCopyClick(Sender: TObject);
    procedure LBoxClick(Sender: TObject);
    procedure BtnFontClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BtnColorClick(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure MemTextChange(Sender: TObject);
    procedure EditLineSpanChange(Sender: TObject);
  private
    FValue: integer;
    FFont: TFont;

    procedure   UpdateSelected();

    procedure   DeleteLayer(const index: integer; L: TLongoLayer);
    procedure   CopyLayer(const index: integer; L: TLongoLayer);

    procedure   SetLayerAlign(const index: integer; L: TLongoLayer);

    procedure   MoveUpLayer(const index: integer; L: TLongoLayer);
    procedure   MoveDownLayer(const index: integer; L: TLongoLayer);

    procedure   SetLayerFont(const index: integer; L: TLongoLayer);
    procedure   SetLayerColor(const index: integer; L: TLongoLayer);
    procedure   SetLayerText(const index: integer; L: TLongoLayer);
    procedure   SetLayerLineSpan(const index: integer; L: TLongoLayer);
    procedure   SetLayerCharSpan(const index: integer; L: TLongoLayer);
  public
    procedure   UpdateItemCount();
  end;

//------------------------------------------------------------------------
//  O̍ĕ`
procedure   KickLayer();

var
  LayerForm: TLayerForm;

implementation

{$R *.dfm}

uses
    FontUnit, ColorUnit, Misc;

//------------------------------------------------------------------------
//  O̍XV
procedure   KickLayer();
begin
    LayerForm.LBox.Repaint();
end;

//------------------------------------------------------------------------
type
    TCustomState = (csNone, csVisibleDown, csParentFontDown);
type
    TMouseStatus = record
        State: TCustomState;
        X: integer;
        Y: integer;
        Index: integer;
    end;
var
    CustomState: TMouseStatus = (State: csNone; X: -1; Y: -1);

//------------------------------------------------------------------------
//  IC[XV
procedure   TLayerForm.UpdateSelected();
var
    i: integer;
begin
    for i := 0 to LBox.Count - 1 do
    begin
        LongoLongo.Selected[i] := LBox.Selected[i];
    end;
//    KickLayer();
    if (LongoLongo.SelCount > 0) then
    begin
        MemText.Text := LongoLongo.SelLayer[0].Text;
        UdLineSpan.Position := LongoLongo.SelLayer[0].LineSpan;
        UdCharSpan.Position := LongoLongo.SelLayer[0].CharSpan;
    end;
end;

//------------------------------------------------------------------------
//  ڐXV(ʃtH[̃R[p)
procedure   TLayerForm.UpdateItemCount();
begin
    LBox.Count := LongoLongo.Count;
end;

//------------------------------------------------------------------------
//  RXgN^
procedure TLayerForm.FormCreate(Sender: TObject);
begin
    Left := MainForm.Left - Width;
    Top := MainForm.Top;
    //  tHgwp
    FFont := TFont.Create;

    SetEditNumbersOnly(EditLineSpan);
    SetEditNumbersOnly(EditCharSpan);
end;

//------------------------------------------------------------------------
//  fXgN^
procedure TLayerForm.FormDestroy(Sender: TObject);
begin
    FFont.Free();
end;

//------------------------------------------------------------------------
//  C[ǉ
procedure TLayerForm.BtnAddClick(Sender: TObject);
var
    index: integer;
begin
    //  C[ǉ
    index := LongoLongo.AddLayers(IntToStr(GetTickCount));
    //  ڐXV
    UpdateItemCount();
    //  I
    LBox.Selected[index] := true;
    UpdateSelected();
    //  ĕ`
    KickMain();
end;

//------------------------------------------------------------------------
//  C[Rs[č쐬
procedure   TLayerForm.CopyLayer(const index: integer; L: TLongoLayer);
begin
    FValue := LongoLongo.CopyLayers(index);
end;
//------------------------------------------------------------------------
procedure TLayerForm.BtnCopyClick(Sender: TObject);
begin
    //  Rs[Ȃ疳
    if (LBox.SelCount = 0) then exit;
    FValue := 0;
    //  IĂzSRs[
    VisitSelectedLayer(true, CopyLayer);
    //  ڐXV
    UpdateItemCount();
    //  I
    LBox.Selected[FValue] := true;
    UpdateSelected();
    //  ĕ`
    KickMain();
end;

//------------------------------------------------------------------------
//  C[폜
procedure   TLayerForm.DeleteLayer(const index: integer; L: TLongoLayer);
begin
    //  C[폜
    LongoLongo.RemoveLayers(index);
    //  Xg{bNXACe폜
    LBox.Items.Delete(index);
    //  CfbNXL^
    FValue := index;
end;
//------------------------------------------------------------------------
procedure TLayerForm.BtnDelClick(Sender: TObject);
begin
    FValue := -1;
    //  폜
    VisitSelectedLayer(false, DeleteLayer);
    //  ڐXV
    LBox.Count := LongoLongo.Count;
    //  ύX
    if (FValue <> -1) then
    begin
        //  Iʒu
        Dec(FValue);
        if (FValue < 0) then FValue := 0;
        if (LBox.Count > FValue) then LBox.Selected[FValue] := true;
        UpdateSelected();
        //  ĕ`
        KickMain();
    end;
end;

//------------------------------------------------------------------------
//  Ɉړ
procedure   TLayerForm.MoveUpLayer(const index: integer; L: TLongoLayer);
begin
    //  C[NXړ
    LongoLongo.MoveLayers(index, index - 1);
    //  XgACeړ
    LBox.Items.Move(index, index - 1);
    //  I󋵈ړ
    LBox.Selected[index - 1] := true;
    LBox.Selected[index] := false;
end;
//------------------------------------------------------------------------
procedure TLayerForm.BtnUpClick(Sender: TObject);
begin
    //  2Ȃבւ
    if (LBox.Count < 2) then exit;
    //  擪IĂ疳
    if (LBox.Selected[0]) then exit;
    //  ړ
    VisitSelectedLayer(true, MoveUpLayer);
    //  ĕ`
    KickMain();
end;

//------------------------------------------------------------------------
//  Ɉړ
procedure   TLayerForm.MoveDownLayer(const index: integer; L: TLongoLayer);
begin
    //  C[NXړ
    LongoLongo.MoveLayers(index, index + 1);
    //  XgACeړ
    LBox.Items.Move(index, index + 1);
    //  I󋵈ړ
    LBox.Selected[index + 1] := true;
    LBox.Selected[index] := false;
end;
//------------------------------------------------------------------------
procedure TLayerForm.BtnDownClick(Sender: TObject);
begin
    //  2Ȃבւ
    if (LBox.Count < 2) then exit;
    //  IĂ疳
    if (LBox.Selected[LBox.Count - 1]) then exit;
    //  ړ
    VisitSelectedLayer(false, MoveDownLayer);
    //  ĕ`
    KickMain();
end;

//------------------------------------------------------------------------
//  ʒu
procedure   TLayerForm.SetLayerAlign(const index: integer; L: TLongoLayer);
begin
    L.SetPosition(TTextAlign(FValue));
end;
//------------------------------------------------------------------------
procedure TLayerForm.BtnLTClick(Sender: TObject);
begin
    FValue := TComponent(Sender).Tag;
    //  IĂ邩
    if (LBox.Count = 0) then exit;
    //  ʒu
    VisitSelectedLayer(true, SetLayerAlign);
    //  ĕ`
    KickMain();
end;

//------------------------------------------------------------------------
//  `
procedure TLayerForm.LBoxDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
const
    CHECK_BTN_HEIGHT = 14;
var
    RectH, MarginB, MarginX, MarginY: integer;
    R1, R1B, R2, R2B, R3, R3T: TRect;
    L: TLongoLayer;
    C: TCanvas;
    DfcState: Cardinal;
    BackColor, ForeColor: TColor;
begin
    //  Zk̂߂̎o
    C := TListBox(Control).Canvas;
    L := LongoLongo.Layer[Index];

    //  ͈͂̍
    RectH := Rect.Bottom - Rect.Top;
    //  `FbN{bNX`p̃}[W
    MarginB := (RectH - CHECK_BTN_HEIGHT) div 2;

    //  [
    R1 := Rect;
    R1.Right := R1.Left + RectH;
    //  [̂ɒ
    R1B := Bounds(R1.Left + MarginB, R1.Top + MarginB, CHECK_BTN_HEIGHT, CHECK_BTN_HEIGHT);

    //  ^
    R2 := Rect;
    R2.Left := R1.Right;
    R2.Right := R2.Left + RectH;
    //  ^̂ɒ
    R2B := Bounds(R2.Left + MarginB, R2.Top + MarginB, CHECK_BTN_HEIGHT, CHECK_BTN_HEIGHT);

    //  c
    R3 := Rect;
    R3.Left := R2.Right;
    //  ĉɒ
    MarginX := 3;
    MarginY := (RectH - C.TextHeight('Wg')) div 2;
    R3T := R3;
    R3T.Left := R3T.Left + MarginX;
    R3T.Top  := R3T.Top + MarginY; 

    //  tH[JX̂Ƃ̔wiF
    if (odFocused in State) then
    begin
        //  IĂ
        if (odSelected in State) then
        begin
            BackColor := RGB(110,136,206);
            ForeColor := RGB(255,255,255);
        end
        //  ȊO
        else
        begin
            BackColor := RGB(240,240,240);
            ForeColor := RGB(000,000,000);
        end;
    end
    //  tH[JX̖Ƃ̔wiF
    else
    begin
        //  IĂ
        if (odSelected in State) then
        begin
            BackColor := RGB(010,036,106);
            ForeColor := RGB(255,255,255);
        end
        //  ȊO
        else
        begin
            BackColor := RGB(255,255,255);
            ForeColor := RGB(000,000,000);
        end;
    end;

    //  wihԂ
    C.Brush.Color := BackColor;
    C.Font.Color := ForeColor;
    C.FillRect(Rect);

    //  Visible\
    DfcState := DFCS_BUTTONCHECK + DFCS_FLAT;
    //  `FbN\
    if (L.Visible) then
    begin
        Inc(DfcState, DFCS_CHECKED);
    end;
    //  ؂ւ
    if (CustomState.State = csVisibleDown) and (CustomState.Index = Index) then
    begin
        Inc(DfcState, DFCS_INACTIVE);
    end;
    //  `
    DrawFrameControl(C.Handle, R1B, DFC_BUTTON, DfcState);

    //  ParentFont\
    DfcState := DFCS_BUTTONCHECK + DFCS_FLAT;
    //  `FbN\
    if (L.ParentFont) then
    begin
        Inc(DfcState, DFCS_CHECKED);
    end;
    //  ؂ւ
    if (CustomState.State = csParentFontDown) and (CustomState.Index = Index) then
    begin
        Inc(DfcState, DFCS_INACTIVE);
    end;
    //  `
    DrawFrameControl(C.Handle, R2B, DFC_BUTTON, DfcState);

    //  eLXg\
    C.TextRect(R3, R3T.Left, R3T.Top, L.Text);

//DEBUGDEBUGDEBUGDEBUG
//    if (L.Selected) then C.TextRect(R3, R3T.Left, R3T.Top, 'Y ' + L.Text)
//                    else C.TextRect(R3, R3T.Left, R3T.Top, 'N ' + L.Text);
//DEBUGDEBUGDEBUGDEBUG

    //  tH[JX_
    if (odFocused in State) then
    begin
        C.DrawFocusRect(Rect);
    end;
end;

//------------------------------------------------------------------------
//  `FbN{bNXH
procedure TLayerForm.LBoxMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
    index, RectH: integer;
    Rect, R1, R2, R3: TRect;
    P: TPoint;
begin
    //  󋵃tONA
    CustomState.State := csNone;

    //  ʒu𒲂ׂ
    P := Point(X, Y);
    index := LBox.ItemAtPos(P, true);
    if (index = -1) then exit;
    Rect := LBox.ItemRect(index);

    //  ͈͂̍
    RectH := Rect.Bottom - Rect.Top;

    //  [
    R1 := Rect;
    R1.Right := R1.Left + RectH;

    //  ^
    R2 := Rect;
    R2.Left := R1.Right;
    R2.Right := R2.Left + RectH;

    //  c
    R3 := Rect;
    R3.Left := R2.Right;

    //  Visible ؂ւ
    if (PtInRect(R1, P)) then
    begin
        CustomState.State := csVisibleDown;
        CustomState.X := X;
        CustomState.Y := Y;
        CustomState.Index := index;
        InvalidateRect(LBox.Handle, @R1, true);
    end
    //  ParentFont ؂ւ
    else if (PtInRect(R2, P)) then
    begin
        CustomState.State := csParentFontDown;
        CustomState.X := X;
        CustomState.Y := Y;
        CustomState.Index := index;
        InvalidateRect(LBox.Handle, @R2, true);
    end
    //  ̑
    else
    begin
        InvalidateRect(LBox.Handle, @Rect, true);
    end;

end;

//------------------------------------------------------------------------
//  `FbN{bNXH
procedure TLayerForm.LBoxMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
    RectH: integer;
    Rect, R1, R2, R3: TRect;
    P: TPoint;
begin
    //  ʒu𒲂ׂ
    P := Point(X, Y);

    //  Ώۂ𒲂ׂ
    Rect := LBox.ItemRect(CustomState.Index);

    //  ͈͂̍
    RectH := Rect.Bottom - Rect.Top;

    //  [
    R1 := Rect;
    R1.Right := R1.Left + RectH;

    //  ^
    R2 := Rect;
    R2.Left := R1.Right;
    R2.Right := R2.Left + RectH;

    //  c
    R3 := Rect;
    R3.Left := R2.Right;

    //  ƓꏊȂ瑀

    //  Visible ؂ւ
    if (PtInRect(R1, P)) and (CustomState.State = csVisibleDown) then
    begin
        LongoLongo.LayerVisible[CustomState.Index]
            := not LongoLongo.LayerVisible[CustomState.Index];
        InvalidateRect(LBox.Handle, @R1, true);
        KickMain();
    end
    //  ParentFont ؂ւ
    else if (PtInRect(R2, P)) and (CustomState.State = csParentFontDown) then
    begin
        LongoLongo.LayerParentFont[CustomState.Index]
            := not LongoLongo.LayerParentFont[CustomState.Index];
        InvalidateRect(LBox.Handle, @R2, true);
        KickMain();
    end
    //  ̑
    else
    begin
        InvalidateRect(LBox.Handle, @Rect, true);
    end;


    //  NA
    CustomState.State := csNone;

end;

//------------------------------------------------------------------------
//  IC[XV
procedure TLayerForm.LBoxClick(Sender: TObject);
begin
    UpdateSelected();
end;

//------------------------------------------------------------------------
//  Kp
procedure   TLayerForm.SetLayerFont(const index: integer; L: TLongoLayer);
begin
    L.ParentFont := false;
    L.Font.Assign(FFont);
end;

//------------------------------------------------------------------------
//  tHgI
procedure TLayerForm.BtnFontClick(Sender: TObject);
var
    L: TLongoLayer;
begin
    if (LongoLongo.SelCount = 0) then exit;
    L := LongoLongo.SelLayer[0];

    FFont.Assign(L.Font);

    if (ChooseFont(Self, FFont)) then
    begin
        VisitSelectedLayer(true, SetLayerFont);
        KickLayer();
        KickMain();
    end;
end;

//------------------------------------------------------------------------
//  Kp
procedure   TLayerForm.SetLayerColor(const index: integer; L: TLongoLayer);
begin
    L.ParentFont := false;
    L.Font.Color := TColor(FValue);
end;

//------------------------------------------------------------------------
//  FI
procedure TLayerForm.BtnColorClick(Sender: TObject);
var
    C: TColor;
    L: TLongoLayer;
begin
    if (LongoLongo.SelCount = 0) then exit;
    L := LongoLongo.SelLayer[0];

    C := L.Font.Color;
    if (ChooseColor(Self, C)) then
    begin
        FValue := C;
        VisitSelectedLayer(true, SetLayerColor);
        KickLayer();
        KickMain();
    end;
end;

//------------------------------------------------------------------------
//  x[XtHgI
procedure TLayerForm.BitBtn1Click(Sender: TObject);
begin
//DEBUGDEBUGDEBUGDEBUG
    FFont.Assign(LongoLongo.Font);
    if (ChooseFont(Self, FFont)) then
    begin
        LongoLongo.Font := FFont;
        KickMain();
    end;
//DEBUGDEBUGDEBUGDEBUG
end;

//------------------------------------------------------------------------
//  x[XtHgFI
procedure TLayerForm.BitBtn2Click(Sender: TObject);
var
    C: TColor;
begin
//DEBUGDEBUGDEBUGDEBUG
    C := LongoLongo.Font.Color;
    if (ChooseColor(Self, C)) then
    begin
        LongoLongo.Font.Color := C;
        KickMain();
    end;
//DEBUGDEBUGDEBUGDEBUG
end;

//------------------------------------------------------------------------
//  eLXgύXKp
procedure   TLayerForm.SetLayerText(const index: integer; L: TLongoLayer);
begin
    L.Text := MemText.Text;
end;

//------------------------------------------------------------------------
//  eLXgύX
procedure TLayerForm.MemTextChange(Sender: TObject);
begin
    VisitSelectedLayer(true, SetLayerText);
    KickLayer();
    KickMain();
end;

//------------------------------------------------------------------------
//  sԁEԕύX
procedure   TLayerForm.SetLayerLineSpan(const index: integer; L: TLongoLayer);
begin
    L.LineSpan := FValue;
end;

//------------------------------------------------------------------------
//  sԁEԕύX
procedure   TLayerForm.SetLayerCharSpan(const index: integer; L: TLongoLayer);
begin
    L.CharSpan := FValue;
end;

//------------------------------------------------------------------------
//  sԁEԕύX
procedure TLayerForm.EditLineSpanChange(Sender: TObject);
var
    E: TEdit;
begin
    E := TEdit(Sender);

    //  sԂ̏ꍇ
    if (E = EditLineSpan) then
    begin
        if (StrToIntDef(E.Text, UdLineSpan.Min - 1) < UdLineSpan.Min) then
        begin
            E.Text := IntToStr(UdLineSpan.Position);
        end;
        FValue := UdLineSpan.Position;
        VisitSelectedLayer(true, SetLayerLineSpan);
    end
    //  Ԃ̏ꍇ
    else if (E = EditCharSpan) then
    begin
        if (StrToIntDef(E.Text, UdCharSpan.Min - 1) < UdCharSpan.Min) then
        begin
            E.Text := IntToStr(UdCharSpan.Position);
        end;
        FValue := UdCharSpan.Position;
        VisitSelectedLayer(true, SetLayerCharSpan);
    end;

    KickMain();
end;

end.


