unit Longo;

interface

uses
    Windows, Classes, Contnrs, SysUtils, StrUtils, Graphics;

type
    //------------------------------------------------------------------------
    //  ʒuw
    TTextAlign = (
        taLeftTop    = 0, taCenterTop    = 1, taRightTop    = 2,
        taLeftCenter = 3, taCenterCenter = 4, taRightCenter = 5,
        taLeftBottom = 6, taCenterBottom = 7,taRightBottom  = 8
    );
    //------------------------------------------------------------------------
    TLongoLongo = class;
    //------------------------------------------------------------------------
    //  C[NX
    TLongoLayer = class
    private
        FVisible: Boolean;
        FText: string;
        FName: string;

        FParent: TLongoLongo;
        FFont: TFont;
        FParentFont: Boolean;

        FPosition: TPoint;
        FLineSpan: integer;     //  s
        FCharSpan: integer;     //  
        FTextHeight: integer;   //  sԁEsl
        FTextWidth: integer;    //  ԁEsl  

        FSelected: Boolean;

        procedure   DrawLine(const PosX, PosY: integer; const S: string; Canvas: TCanvas);
        procedure   Draw(Canvas: TCanvas);

        procedure   SetLeftTop(const R: TRect; Canvas: TCanvas);
        procedure   SetLeftCenter(const R: TRect; Canvas: TCanvas);
        procedure   SetLeftBottom(const R: TRect; Canvas: TCanvas);
        procedure   SetCenterTop(const R: TRect; Canvas: TCanvas);
        procedure   SetCenterCenter(const R: TRect; Canvas: TCanvas);
        procedure   SetCenterBottom(const R: TRect; Canvas: TCanvas);
        procedure   SetRightTop(const R: TRect; Canvas: TCanvas);
        procedure   SetRightCenter(const R: TRect; Canvas: TCanvas);
        procedure   SetRightBottom(const R: TRect; Canvas: TCanvas);
        procedure   SetSelected(Value: Boolean);

        procedure   SetText(const Value: string);
        function    GetText(): string;

        function    TextHeight(Canvas: TCanvas): integer;
        function    TextWidth1(const S: string; Canvas: TCanvas): integer;
        function    TextWidth(Canvas: TCanvas): integer;
    public
        constructor Create(const S: string; Parent: TLongoLongo);
        destructor  Destroy(); override;
        procedure   Assign(L: TLongoLayer);

        procedure   SetPosition(const Align: TTextAlign);
//        procedure   AbsoluteMove(const X, Y: integer);
        procedure   RelativeMove(const DX, DY: integer);

        property    Visible: Boolean read FVisible write FVisible;
        property    Text: string read GetText write SetText;
        property    Name: string read FName write FName;
        property    Font: TFont read FFont write FFont;
        property    ParentFont: Boolean read FParentFont write FParentFont;
        property    Position: TPoint read FPosition write FPosition;
        property    LineSpan: integer read FLineSpan write FLineSpan;
        property    CharSpan: integer read FCharSpan write FCharSpan;
        property    Selected: Boolean read FSelected write SetSelected;
    end;
    //------------------------------------------------------------------------
    //  C[ǗNX
    TLongoLongo = class
    private
        FBitmap: TBitmap;
        FLayers: TObjectList;
        FSelLayers: TObjectList;
        FWidth: integer;
        FHeight: integer;
        FWorkFont: TFont;

        procedure   SetLayerVisible(index: integer; const Value: Boolean);
        function    GetLayerVisible(index: integer): Boolean;
        procedure   SetLayerParentFont(index: integer; const Value: Boolean);
        function    GetLayerParentFont(index: integer): Boolean;
        procedure   SetLayerText(index: integer; const Value: string);
        function    GetLayerText(index: integer): string;
        function    GetLayer(index: integer): TLongoLayer;
        function    GetLayerCount(): integer;
        procedure   SetLayerSelected(index: integer; const Value: Boolean);
        function    GetLayerSelected(index: integer): Boolean;
        function    GetSelLayer(index: integer): TLongoLayer;
        function    GetSelLayerCount(): integer;

        procedure   SetBaseFont(F: TFont);
        function    GetBaseFont(): TFont;
    public
        procedure   FontBackUp();
        procedure   FontRestore();

        procedure   ChangeWidth(const W: integer);
        procedure   ChangeHeight(const H: integer);
        procedure   ChangeSize(const W, H: integer);

        constructor Create(const W, H: integer);
        destructor  Destroy(); override;

        function    AddLayers(const N: string): integer;
        function    CopyLayers(const Src: integer): integer;
        procedure   RemoveLayers(const index: integer);
        procedure   MoveLayers(const OldPos, NewPos: integer);
        procedure   Draw(Canvas: TCanvas);

        procedure   SetLayerPosition(const index: integer; const Align: TTextAlign);
//        procedure   LayerAbsoluteMove(const index, X, Y: integer);
        procedure   LayerRelativeMove(const index, DX, DY: integer);

        property    LayerVisible[index: integer]: Boolean read GetLayerVisible write SetLayerVisible;
        property    LayerParentFont[index: integer]: Boolean read GetLayerParentFont write SetLayerParentFont;
        property    LayerText[index: integer]: string read GetLayerText write SetLayerText;

        property    Layer[index: integer]: TLongoLayer read GetLayer;
        property    Width: integer read FWidth write FWidth;
        property    Height: integer read FHeight write FHeight;
        property    Bitmap: TBitmap read FBitmap write FBitmap;
        property    Count: integer read GetLayerCount;
        property    SelLayers: TObjectList read FSelLayers write FSelLayers;
        property    SelLayer[index: integer]: TLongoLayer read GetSelLayer;
        property    Selected[index: integer]: Boolean read GetLayerSelected write SetLayerSelected;
        property    SelCount: integer read GetSelLayerCount;

        property    Font: TFont read GetBaseFont write SetBaseFont;
    end;
    //------------------------------------------------------------------------
    //  IC[pR[obN֐
    TLayerVisitorProc = procedure(const index: integer; L: TLongoLayer) of object;
    //------------------------------------------------------------------------
    //  IC[֐
    function    VisitSelectedLayer(const Order: Boolean; Proc: TLayerVisitorProc): integer;

var
    LongoLongo: TLongoLongo = nil;

implementation

//------------------------------------------------------------------------
//  IC[񂷂
function    VisitSelectedLayer(const Order: Boolean; Proc: TLayerVisitorProc): integer;
var
    i: integer;
begin
    //  
    Result := LongoLongo.SelCount;

    //  O
    if (Order) then
    begin
        for i := 0 to LongoLongo.Count - 1 do
        begin
            if (LongoLongo.Selected[i]) then
            begin
                Proc(i, LongoLongo.Layer[i]);
            end;
        end;
    end
    //  납
    else
    begin
        for i := LongoLongo.Count - 1 downto 0 do
        begin
            if (LongoLongo.Selected[i]) then
            begin
                Proc(i, LongoLongo.Layer[i]);
            end;
        end;
    end;
end;

//************************************************************************
//  RXgN^
constructor TLongoLayer.Create(const S: string; Parent: TLongoLongo);
begin
    //  Oݒ
    Name := FormatDateTime('yyyymmdd"_"hhnnsszzz"_%10.10d"', Now());
    Name := Format(Name, [GetTickCount()]);
    //  e
    FParent := Parent;
    //  Ƃ肠\
    Visible := true;
    //  ݒ
    Text := S;
    //  ʒu
    Position := Point(0, 0);
    //  sԁE
    FLineSpan := 0;
    FCharSpan := 0;
    //  {tHggp
    ParentFont := true;
    //  ƎtHgp
    Font := TFont.Create();
    //  {tHgRs[
    Font.Assign(FParent.FBitmap.Canvas.Font);
end;
//------------------------------------------------------------------------
//  fXgN^
destructor  TLongoLayer.Destroy();
begin
    FParent.SelLayers.Remove(Self);
    Font.Free();
end;
//------------------------------------------------------------------------
//  C[vpeBRs[
procedure   TLongoLayer.Assign(L: TLongoLayer);
begin
    Visible := L.Visible;
//    Name := L.Name;
    Text := L.Text;
    ParentFont := L.ParentFont;
    Font.Assign(L.Font);
    Position := L.Position;
    FLineSpan := L.LineSpan;
    FCharSpan := L.CharSpan;
end;
//------------------------------------------------------------------------
//  
procedure   TLongoLayer.SetLeftTop(const R: TRect; Canvas: TCanvas);
begin
    FPosition.X := R.Left;
    FPosition.Y := R.Top;
end;
//------------------------------------------------------------------------
procedure   TLongoLayer.SetLeftCenter(const R: TRect; Canvas: TCanvas);
begin
    FPosition.X := R.Left;
    FPosition.Y := R.Top + (R.Bottom - R.Top - FTextHeight) div 2;
end;
//------------------------------------------------------------------------
procedure   TLongoLayer.SetLeftBottom(const R: TRect; Canvas: TCanvas);
begin
    FPosition.X := R.Left;
    FPosition.Y := R.Top + (R.Bottom - FTextHeight);
end;
//------------------------------------------------------------------------
procedure   TLongoLayer.SetCenterTop(const R: TRect; Canvas: TCanvas);
begin
    FPosition.X := R.Left + (R.Right - R.Left - FTextWidth) div 2;
    FPosition.Y := R.Top;
end;
//------------------------------------------------------------------------
procedure   TLongoLayer.SetCenterCenter(const R: TRect; Canvas: TCanvas);
begin
    FPosition.X := R.Left + (R.Right - R.Left - FTextWidth) div 2;
    FPosition.Y := R.Top + (R.Bottom - R.Top - FTextHeight) div 2;
end;
//------------------------------------------------------------------------
procedure   TLongoLayer.SetCenterBottom(const R: TRect; Canvas: TCanvas);
begin
    FPosition.X := R.Left + (R.Right - R.Left - FTextWidth) div 2;
    FPosition.Y := R.Top + (R.Bottom - FTextHeight);
end;
//------------------------------------------------------------------------
procedure   TLongoLayer.SetRightTop(const R: TRect; Canvas: TCanvas);
begin
    FPosition.X := R.Left + (R.Right - FTextWidth);
    FPosition.Y := R.Top;
end;
//------------------------------------------------------------------------
procedure   TLongoLayer.SetRightCenter(const R: TRect; Canvas: TCanvas);
begin
    FPosition.X := R.Left + (R.Right - FTextWidth);
    FPosition.Y := R.Top + (R.Bottom - R.Top - FTextHeight) div 2;
end;
//------------------------------------------------------------------------
procedure   TLongoLayer.SetRightBottom(const R: TRect; Canvas: TCanvas);
begin
    FPosition.X := R.Left + (R.Right - FTextWidth);
    FPosition.Y := R.Top + (R.Bottom - FTextHeight);
end;
//------------------------------------------------------------------------
//  `掞̍
function    TLongoLayer.TextHeight(Canvas: TCanvas): integer;
var
    Len0, Len1, LineCount: integer;
begin
    //  ̒
    Len0 := Length(FText);
    //  s΂
    Len1 := Length(StringReplace(FText, #255#255, '', [rfReplaceAll]));
    //   = s
    LineCount := (Len0 - Len1) div 2 + 1;
    //   = (1s̍ + s) * (s - 1) + Ō̍s̍ 
    Result := (Canvas.TextHeight(FText) + LineSpan) * (LineCount - 1) + Canvas.TextHeight(FText); 
end;
//------------------------------------------------------------------------
//  `掞̂1s̕
function    TLongoLayer.TextWidth1(const S: string; Canvas: TCanvas): integer;
var
    i: integer;
    str: string;
begin
    Result := 0;

    for i := 1 to Length(S) do
    begin
        //  2oCg1oCg
        if (ByteType(S, i) = mbLeadByte) then
        begin
            str := S[i];
            continue;
        end
        //  2oCg2oCg
        else if (ByteType(S, i) = mbTrailByte) then
        begin
            str := str + S[i];
        end
        //  1oCg
        else
        begin
            str := S[i];
        end;
        //  ʒu߂
        Inc(Result, Canvas.TextWidth(str) + CharSpan);
    end;

    Dec(Result, CharSpan);
end;
//------------------------------------------------------------------------
//  `掞̕
function    TLongoLayer.TextWidth(Canvas: TCanvas): integer;
var
    index: integer;
    s, s1: string;
    TmpLen: integer;
begin
    Result := 0;

    //  1s`FbN
    s := FText;
    while (s <> '') do
    begin
        //  sT
        index := Pos(#255#255, s);
        //  Ȃ΂ŏI
        if (index = 0) then
        begin
            s1 := s;
            s := '';
        end
        //  
        else
        begin
            s1 := Copy(s, 1, index - 1);
            Delete(s, 1, index + 1); 
        end;
        
        TmpLen := TextWidth1(s1, Canvas);
        if (Result < TmpLen) then Result := TmpLen; 
    end;
end;
//------------------------------------------------------------------------
procedure   TLongoLayer.SetPosition(const Align: TTextAlign);
var
    R: TRect;
    C: TCanvas;
begin
    C := FParent.Bitmap.Canvas;

    //  `Ώۂ̋`
    R := Rect(0, 0, FParent.Width, FParent.Height);

    //  C[tHgݒ
    if (not ParentFont) then
    begin
        FParent.FontBackUp();
        C.Font.Assign(FFont);
    end;

    //  s̍
    FTextHeight := TextHeight(C);

    //  Ƃ̍Ls߂
    FTextWidth := TextWidth(C);
    
    //  ʒu
    if (Align = taLeftTop) then SetLeftTop(R, C)
    else if (Align = taLeftCenter) then SetLeftCenter(R, C)
    else if (Align = taLeftBottom) then SetLeftBottom(R, C)

    else if (Align = taCenterTop) then SetCenterTop(R, C)
    else if (Align = taCenterCenter) then SetCenterCenter(R, C)
    else if (Align = taCenterBottom) then SetCenterBottom(R, C)

    else if (Align = taRightTop) then SetRightTop(R, C)
    else if (Align = taRightCenter) then SetRightCenter(R, C)
//    else if (Align = taRightBottom) then SetRightBottom(R, C);
    else SetRightBottom(R, C);

    //  tHgA
    if (not ParentFont) then
    begin
        FParent.FontRestore();
    end;
end;
//------------------------------------------------------------------------
//  eLXgݒ
procedure   TLongoLayer.SetText(const Value: string);
begin
    //  [΂
    FText := Trim(Value);
    //  suĂ
    FText := StringReplace(FText, #13#10, #255#255, [rfReplaceAll]);
end;
//------------------------------------------------------------------------
//  eLXg擾
function    TLongoLayer.GetText(): string;
begin
    Result := StringReplace(FText, #255#255, #13#10, [rfReplaceAll]);
end;
//------------------------------------------------------------------------
//  1s`
procedure   TLongoLayer.DrawLine(const PosX, PosY: integer; const S: string; Canvas: TCanvas);
var
    i, x: integer;
    str: string;
begin
    x := PosX;

    for i := 1 to Length(S) do
    begin
        //  2oCg1oCg
        if (ByteType(S, i) = mbLeadByte) then
        begin
            str := S[i];
            continue;
        end
        //  2oCg2oCg
        else if (ByteType(S, i) = mbTrailByte) then
        begin
            str := str + S[i];
        end
        //  1oCg
        else
        begin
            str := S[i];
        end;
        //  `
        Canvas.TextOut(x, PosY, str);
        //  ʒu߂
        Inc(x, Canvas.TextWidth(str) + CharSpan);
    end;
end;
//------------------------------------------------------------------------
//  `
procedure   TLongoLayer.Draw(Canvas: TCanvas);
var
    index, LineHeight: integer;
    PosX, PosY: integer;
    s, s1: string;
begin
    //  Ă܂
    if (not Visible) then exit;

    //  C[tHgݒ
    if (not ParentFont) then
    begin
        FParent.FontBackUp();
        Canvas.Font.Assign(FFont);
    end;

    //  1s̍
    LineHeight := Canvas.TextHeight(FText);

    //  `Jnʒu
    PosX := Position.X;
    PosY := Position.Y;

    //  1s`
    s := FText;
    while (s <> '') do
    begin
        //  sT
        index := Pos(#255#255, s);
        //  Ȃ΂ŏI
        if (index = 0) then
        begin
            s1 := s;
            s := '';
        end
        //  
        else
        begin
            s1 := Copy(s, 1, index - 1);
            Delete(s, 1, index + 1); 
        end;
        DrawLine(PosX, PosY, s1, Canvas);
        Inc(PosY, LineHeight + LineSpan);
    end;

    //  tHgA
    if (not ParentFont) then
    begin
        FParent.FontRestore();
    end;
end;
//------------------------------------------------------------------------
//  IԕύX
procedure   TLongoLayer.SetSelected(Value: Boolean);
begin
    //  ԂύX̎
    if (FSelected <> Value) then
    begin
        //  I
        if (Value) then
        begin
            FParent.SelLayers.Add(Self);
        end
        //  I
        else
        begin
            FParent.SelLayers.Remove(Self);
        end;
        FSelected := Value;
    end;
end;
{
//------------------------------------------------------------------------
//  ړ
procedure   TLongoLayer.AbsoluteMove(const X, Y: integer);
begin
    FPosition.X := X;
    FPosition.Y := Y;
end;
}
//------------------------------------------------------------------------
//  炵
procedure   TLongoLayer.RelativeMove(const DX, DY: integer);
begin
    FPosition.X := FPosition.X + DX;
    FPosition.Y := FPosition.Y + DY; 
end;

//************************************************************************
//  RXgN^
constructor TLongoLongo.Create(const W, H: integer);
begin
    //  ƍ
    Width := W;
    Height := H;
    //  `prbg}bv
    FBitmap := TBitmap.Create;
    FBitmap.PixelFormat := pf24bit;
    FBitmap.Width := Width;
    FBitmap.Height := Height;
    //  ʃtHg
    FWorkFont := TFont.Create();
    //  C[Xg
    FLayers := TObjectList.Create(true);
    //  I𒆃C[Xg
    FSelLayers := TObjectList.Create(false);
end;
//------------------------------------------------------------------------
//  fXgN^
destructor  TLongoLongo.Destroy();
begin
    FLayers.Free;
    FSelLayers.Free;
    FBitmap.Free;
    FWorkFont.Free;
end;
//------------------------------------------------------------------------
//  tHgobNAbv
procedure   TLongoLongo.FontBackUp();
begin
    FWorkFont.Assign(FBitmap.Canvas.Font);
end;
//------------------------------------------------------------------------
//  tHgXgA
procedure   TLongoLongo.FontRestore();
begin
    FBitmap.Canvas.Font.Assign(FWorkFont);
end;
//------------------------------------------------------------------------
//  {tHg؂ւ
procedure   TLongoLongo.SetBaseFont(F: TFont);
begin
//DEBUGDEBUGDEBUGDEBUG
    FBitmap.Canvas.Font.Assign(F);
//DEBUGDEBUGDEBUGDEBUG
end;
//------------------------------------------------------------------------
function    TLongoLongo.GetBaseFont(): TFont;
begin
//DEBUGDEBUGDEBUGDEBUG
    Result := FBitmap.Canvas.Font;
//DEBUGDEBUGDEBUGDEBUG
end;
//------------------------------------------------------------------------
//  C[ǉ
function    TLongoLongo.AddLayers(const N: string): integer;
var
    L: TLongoLayer;
begin
    L := TLongoLayer.Create(N, Self);
    Result := FLayers.Add(L);
end;
//------------------------------------------------------------------------
//  C[
function    TLongoLongo.CopyLayers(const Src: integer): integer;
var
    L: TLongoLayer;
begin
    L := TLongoLayer.Create('', Self);
    L.Assign(GetLayer(Src));
    Result := FLayers.Add(L);
end;
//------------------------------------------------------------------------
//  C[폜
procedure   TLongoLongo.RemoveLayers(const index: integer);
begin
    FLayers.Delete(index);
end;
//------------------------------------------------------------------------
//  ړ
procedure   TLongoLongo.MoveLayers(const OldPos, NewPos: integer);
begin
    FLayers.Move(OldPos, NewPos);
end;
//------------------------------------------------------------------------
//  ύX
procedure   TLongoLongo.ChangeWidth(const W: integer);
begin
    Width := W;
    FBitmap.Width := Width;
end;
//------------------------------------------------------------------------
//  ύX
procedure   TLongoLongo.ChangeHeight(const H: integer);
begin
    Height := H;
    FBitmap.Height := Height;
end;
//------------------------------------------------------------------------
//  TCYύX
procedure   TLongoLongo.ChangeSize(const W, H: integer);
begin
    ChangeWidth(W);
    ChangeHeight(H);
end;
//------------------------------------------------------------------------
//  C[̖擾
function    TLongoLongo.GetLayerCount(): integer;
begin
    Result := FLayers.Count;
end;
//------------------------------------------------------------------------
//  C[擾
function    TLongoLongo.GetLayer(index: integer): TLongoLayer;
begin
    Result := TLongoLayer(FLayers[index]);
end;
//------------------------------------------------------------------------
//  `
procedure   TLongoLongo.Draw(Canvas: TCanvas);
var
    i: integer;
begin
    FBitmap.Canvas.Brush.Style := bsSolid;
    FBitmap.Canvas.FillRect(Rect(0, 0, Width, Height));
    FBitmap.Canvas.Brush.Style := bsClear;
    for i := FLayers.Count - 1 downto 0 do
    begin
        GetLayer(i).Draw(FBitmap.Canvas);
    end;
    BitBlt(Canvas.Handle, 0, 0, Width, Height, FBitmap.Canvas.Handle, 0, 0, SRCCOPY);
end;
//------------------------------------------------------------------------
//  C[\/\
procedure   TLongoLongo.SetLayerVisible(index: integer; const Value: Boolean);
begin
    GetLayer(index).Visible := Value;
end;
//------------------------------------------------------------------------
function    TLongoLongo.GetLayerVisible(index: integer): Boolean;
begin
    Result := GetLayer(index).Visible;
end;
//------------------------------------------------------------------------
//  C[tHg{tHggp邩ǂ
procedure   TLongoLongo.SetLayerParentFont(index: integer; const Value: Boolean);
begin
    GetLayer(index).ParentFont := Value;
end;
//------------------------------------------------------------------------
function    TLongoLongo.GetLayerParentFont(index: integer): Boolean;
begin
    Result := GetLayer(index).ParentFont;
end;
//------------------------------------------------------------------------
//  C[
procedure   TLongoLongo.SetLayerText(index: integer; const Value: string);
begin
    GetLayer(index).Text := Value;
end;
//------------------------------------------------------------------------
function    TLongoLongo.GetLayerText(index: integer): string;
begin
    Result := GetLayer(index).Text;
end;
//------------------------------------------------------------------------
//  C[`ʒu
procedure   TLongoLongo.SetLayerPosition(const index: integer; const Align: TTextAlign);
begin
    GetLayer(index).SetPosition(Align);
end;
{
//------------------------------------------------------------------------
//  C[`ʒuw
procedure   TLongoLongo.LayerAbsoluteMove(const index, X, Y: integer);
begin
    GetLayer(index).AbsoluteMove(X, Y);
end;
}
//------------------------------------------------------------------------
//  C[`ʒu炵
procedure   TLongoLongo.LayerRelativeMove(const index, DX, DY: integer);
begin
    GetLayer(index).RelativeMove(DX, DY);
end;
//------------------------------------------------------------------------
//  C[I
procedure   TLongoLongo.SetLayerSelected(index: integer; const Value: Boolean);
begin
    GetLayer(index).Selected := Value;
end;
//------------------------------------------------------------------------
function    TLongoLongo.GetLayerSelected(index: integer): Boolean;
begin
    Result := GetLayer(index).Selected;
end;
//------------------------------------------------------------------------
//  IC[o
function    TLongoLongo.GetSelLayer(index: integer): TLongoLayer;
begin
    Result := TLongoLayer(FSelLayers[index]);
end;
//------------------------------------------------------------------------
//  I
function    TLongoLongo.GetSelLayerCount(): integer;
begin
    Result := FSelLayers.Count;
end;
//------------------------------------------------------------------------
//------------------------------------------------------------------------
//------------------------------------------------------------------------
//------------------------------------------------------------------------
//------------------------------------------------------------------------
//------------------------------------------------------------------------
//------------------------------------------------------------------------
//------------------------------------------------------------------------
//------------------------------------------------------------------------
//------------------------------------------------------------------------
end.

