unit Misc;

interface

uses
  Windows, SysUtils, Classes, Graphics, StdCtrls, Controls, Grids;

//------------------------------------------------------------------------
//  WEBF
const
    HTML_COLOR: array[0..5,0..5,0..5] of string
    = (
        (
            ('#000000','#000033','#000066','#000099','#0000cc','#0000ff'),
            ('#003300','#003333','#003366','#003399','#0033cc','#0033ff'),
            ('#006600','#006633','#006666','#006699','#0066cc','#0066ff'),
            ('#009900','#009933','#009966','#009999','#0099cc','#0099ff'),
            ('#00cc00','#00cc33','#00cc66','#00cc99','#00cccc','#00ccff'),
            ('#00ff00','#00ff33','#00ff66','#00ff99','#00ffcc','#00ffff')
        ),
        (
            ('#330000','#330033','#330066','#330099','#3300cc','#3300ff'),
            ('#333300','#333333','#333366','#333399','#3333cc','#3333ff'),
            ('#336600','#336633','#336666','#336699','#3366cc','#3366ff'),
            ('#339900','#339933','#339966','#339999','#3399cc','#3399ff'),
            ('#33cc00','#33cc33','#33cc66','#33cc99','#33cccc','#33ccff'),
            ('#33ff00','#33ff33','#33ff66','#33ff99','#33ffcc','#33ffff')
        ),
        (
            ('#660000','#660033','#660066','#660099','#6600cc','#6600ff'),
            ('#663300','#663333','#663366','#663399','#6633cc','#6633ff'),
            ('#666600','#666633','#666666','#666699','#6666cc','#6666ff'),
            ('#669900','#669933','#669966','#669999','#6699cc','#6699ff'),
            ('#66cc00','#66cc33','#66cc66','#66cc99','#66cccc','#66ccff'),
            ('#66ff00','#66ff33','#66ff66','#66ff99','#66ffcc','#66ffff')
        ),
        (
            ('#990000','#990033','#990066','#990099','#9900cc','#9900ff'),
            ('#993300','#993333','#993366','#993399','#9933cc','#9933ff'),
            ('#996600','#996633','#996666','#996699','#9966cc','#9966ff'),
            ('#999900','#999933','#999966','#999999','#9999cc','#9999ff'),
            ('#99cc00','#99cc33','#99cc66','#99cc99','#99cccc','#99ccff'),
            ('#99ff00','#99ff33','#99ff66','#99ff99','#99ffcc','#99ffff')
        ),
        (
            ('#cc0000','#cc0033','#cc0066','#cc0099','#cc00cc','#cc00ff'),
            ('#cc3300','#cc3333','#cc3366','#cc3399','#cc33cc','#cc33ff'),
            ('#cc6600','#cc6633','#cc6666','#cc6699','#cc66cc','#cc66ff'),
            ('#cc9900','#cc9933','#cc9966','#cc9999','#cc99cc','#cc99ff'),
            ('#cccc00','#cccc33','#cccc66','#cccc99','#cccccc','#ccccff'),
            ('#ccff00','#ccff33','#ccff66','#ccff99','#ccffcc','#ccffff')
        ),
        (
            ('#ff0000','#ff0033','#ff0066','#ff0099','#ff00cc','#ff00ff'),
            ('#ff3300','#ff3333','#ff3366','#ff3399','#ff33cc','#ff33ff'),
            ('#ff6600','#ff6633','#ff6666','#ff6699','#ff66cc','#ff66ff'),
            ('#ff9900','#ff9933','#ff9966','#ff9999','#ff99cc','#ff99ff'),
            ('#ffcc00','#ffcc33','#ffcc66','#ffcc99','#ffcccc','#ffccff'),
            ('#ffff00','#ffff33','#ffff66','#ffff99','#ffffcc','#ffffff')
        )
      );

    SAMPLE_TEXT: array[0..2] of string =
    (
        'AaBbYyZz012', 'Symbol012', 'AaA@F'
    ); 
//------------------------------------------------------------------------
procedure   SetEditNumbersOnly(E: TEdit);
//------------------------------------------------------------------------
function    HTMLToColor(const S: string; const Default: TColor = 0): TColor;
procedure   InitHTMLColorMap(G: TStringGrid);
procedure   DrawCrossCursor(C: TCanvas; const X, Y: integer);
procedure   DrawColorMap(B: TBitmap);
procedure   DrawLineDiff(C: TCanvas; const X, Y, DX, DY: integer);
function    ClientRectToScreenRect(ASrcControl: TControl): TRect;
//------------------------------------------------------------------------
function    FontEnumProc(LogFont: PEnumLogFont; TextMetrics: PNewTextMetric;
                FontType: integer; lParam: LPARAM): integer; stdcall;
function    SortFontList(L: TStringList; Index1, Index2: Integer): Integer;
//------------------------------------------------------------------------

implementation

//------------------------------------------------------------------------
//  Delphi Win32 Graphics API t@X P618-
//  R[obN֐
function    FontEnumProc(LogFont: PEnumLogFont; TextMetrics: PNewTextMetric;
                FontType: integer; lParam: LPARAM): integer; stdcall;
var
    FaceName: string;
begin
    //  tHgƃtHg^CvXgɒǉ
    FaceName := TEnumLogFont(LogFont^).elfLogFont.lfFaceName;
    if (Copy(FaceName, 1, 1) <> '@') then
    begin
        TStrings(lParam).AddObject(FaceName, TObject(TextMetrics.tmCharSet));
    end;
    //  񋓌p
    Result := 1;
end;

//------------------------------------------------------------------------
//  ORDER BY CharSet, FontName
function    SortFontList(L: TStringList; Index1, Index2: Integer): Integer;
var
    CharSet1, CharSet2: TFontCharSet;
    Name1, Name2: string;
begin
    //  CharSet Ń\[g
    Charset1 := TFontCharSet(L.Objects[Index1]);
    Charset2 := TFontCharSet(L.Objects[Index2]);
    Result := (Charset1 - Charset2);
    //  FontName Ń\[g
    if (Result = 0) then
    begin
        Name1 := L[Index1];
        Name2 := L[Index2];
        Result := CompareText(Name1, Name2);
    end;
end;

(*
    -----------------------------------------------------------------
    ClientToScreen Rect
    -----------------------------------------------------------------
*)
function    ClientRectToScreenRect(ASrcControl: TControl): TRect;
var
    W, H: integer;
    L, T: integer;
begin
    //  IWi̕ƍ
    with ASrcControl.ClientRect do
    begin
        W := Right - Left;
        H := Bottom - Top;
    end;
    //  IWi̍
    with ASrcControl.ClientOrigin do
    begin
        L := X;
        T := Y;
    end;
    Result := Bounds(L, T, W, H);
end;

(*
    -----------------------------------------------------------------
     MoveTo & LineTo
    -----------------------------------------------------------------
*)
procedure   DrawLineDiff(C: TCanvas; const X, Y, DX, DY: integer);
begin
    C.MoveTo(X, Y);
    C.LineTo(X + DX, Y + DY);
end;

(*
    -----------------------------------------------------------------
    ݈ʒuȂɕ\
    -----------------------------------------------------------------
*)
procedure   DrawCrossCursor(C: TCanvas; const X, Y: integer);
var
    PrePenW: integer;
begin
    PrePenW := C.Pen.Width;
    C.Pen.Width := 3;
    DrawLineDiff(C, X - 4, Y - 0,  2,  0);
    DrawLineDiff(C, X + 2, Y - 0,  2,  0);
    DrawLineDiff(C, X - 0, Y - 4,  0,  2);
    DrawLineDiff(C, X - 0, Y + 2,  0,  2);
    C.Pen.Width := PrePenW;
end;

//------------------------------------------------------------------------
//  F}bv쐬
procedure   DrawColorMap(B: TBitmap);
var
    x: integer;
    L: PByteArray;
begin
    B.Width := 256;
    B.Height := 3;
    B.PixelFormat := pf24bit;

    //  
    L := PByteArray(B.ScanLine[0]);
    for x := 0 to B.Width - 1 do
    begin
        L[x * 3 + 0] := 0;  //  B
        L[x * 3 + 1] := 0;  //  G
        L[x * 3 + 2] := x;  //  R
    end;

    //  
    L := PByteArray(B.ScanLine[1]);
    for x := 0 to B.Width - 1 do
    begin
        L[x * 3 + 0] := 0;  //  B
        L[x * 3 + 1] := x;  //  G
        L[x * 3 + 2] := 0;  //  R
    end;

    //  
    L := PByteArray(B.ScanLine[2]);
    for x := 0 to B.Width - 1 do
    begin
        L[x * 3 + 0] := x;  //  B
        L[x * 3 + 1] := 0;  //  G
        L[x * 3 + 2] := 0;  //  R
    end;
end;

//------------------------------------------------------------------------
//  HTML\̐Fϊ
function    HTMLToColor(const S: string; const Default: TColor = 0): TColor;
begin
    Result := StrToIntDef('$00' + Copy(S, 6, 2) + Copy(S, 4, 2) + Copy(S, 2, 2), Default);
end;

//------------------------------------------------------------------------
//  HTMLFObhɏ
procedure   InitHTMLColorMap(G: TStringGrid);
var
    i, j: integer;
begin
    for i := 0 to 36 - 1 do
    begin
        for j := 0 to 6 - 1 do
        begin
            G.Cells[i, j] := IntToStr(HTMLToColor(HTML_COLOR[i div 6, i mod 6, j]));
//            G.Cells[i, j] := HTML_COLOR[i div 6, i mod 6, j];
        end;
    end;
end;

//------------------------------------------------------------------------
//  TEdit𐔒lp
procedure   SetEditNumbersOnly(E: TEdit);
begin
    SetWindowLong(E.Handle, GWL_STYLE, GetWindowLong(E.Handle, GWL_STYLE) + ES_NUMBER);
end;

end.

